import 'package:rec_api_dart/rec_api_dart.dart';

class Challenge extends Entity {
  final ChallengeStatus status;
  final ChallengeAction action;
  final ChallengeType type;
  final ChallengeStats stats;

  final String title;
  final String description;
  final String coverImage;

  final int threshold;
  final int amountRequired;

  final DateTime startDate;
  final DateTime finishDate;

  final List<Activity> activities;
  final TokenReward tokenReward;

  bool get isAmountChallenge => amountRequired > 0;
  bool get isThresholdChallenge => threshold > 0;

  int get amountRemaining => amountRequired - stats.totalAmount;
  int get txsRemaining => threshold - stats.totalTxs;

  Challenge({
    String? id,
    String? createdAt,
    String? updatedAt,
    required this.status,
    required this.action,
    required this.type,
    required this.title,
    required this.description,
    required this.coverImage,
    required this.threshold,
    required this.amountRequired,
    required this.startDate,
    required this.finishDate,
    required this.activities,
    required this.tokenReward,
    required this.stats,
  }) : super(id, createdAt, updatedAt);

  factory Challenge.fromJson(Map<String, dynamic> json) {
    var activities = <Activity>[];

    if (json['activities'] != null) {
      final jsonActivities = List.from(json['activities']);
      activities = jsonActivities.isNotEmpty
          ? jsonActivities.map<Activity>((el) => Activity.fromJson(el)).toList()
          : <Activity>[];
    }

    return Challenge(
      activities: activities,
      coverImage: json['cover_image'],
      description: json['description'],
      finishDate: DateHelper.safeParse(json['finish_date']),
      startDate: DateHelper.safeParse(json['start_date']),
      status: ChallengeStatus.fromName(json['status']),
      action: ChallengeAction.fromName(json['action']),
      type: ChallengeType.fromName(json['type']),
      threshold: json['threshold'],
      amountRequired: json['amount_required'],
      title: json['title'],
      tokenReward: TokenReward.fromJson(json['token_reward']),
      stats: ChallengeStats.fromJson(json['statistics'] ?? {}),
    );
  }
}

class ChallengeStats {
  final int totalTxs;
  final int totalAmount;

  ChallengeStats({this.totalTxs = 0, this.totalAmount = 0});

  factory ChallengeStats.fromJson([Map<String, dynamic>? json = const {}]) {
    return ChallengeStats(
      totalTxs: json?['total_tx'] ?? 0,
      totalAmount: json?['total_amount'] ?? 0,
    );
  }
}

enum ChallengeAction {
  BUY,
  QUALIFICATION,
  RECHARGE,
  SEND,
  OTHER;

  String get name {
    switch (this) {
      case ChallengeAction.BUY:
        return 'buy';
      case ChallengeAction.QUALIFICATION:
        return 'qualification';
      case ChallengeAction.RECHARGE:
        return 'recharge';
      case ChallengeAction.SEND:
        return 'send';
      default:
        return 'other';
    }
  }

  static ChallengeAction fromName(String name) {
    switch (name) {
      case 'buy':
        return ChallengeAction.BUY;
      case 'qualification':
        return ChallengeAction.QUALIFICATION;
      case 'recharge':
        return ChallengeAction.RECHARGE;
      case 'send':
        return ChallengeAction.SEND;
      default:
        return ChallengeAction.OTHER;
    }
  }
}

enum ChallengeStatus {
  DRAFT,
  CLOSED,
  OPEN,
  SCHEDULED,
  OTHER;

  get name {
    switch (this) {
      case ChallengeStatus.DRAFT:
        return 'draft';
      case ChallengeStatus.CLOSED:
        return 'closed';
      case ChallengeStatus.OPEN:
        return 'open';
      case ChallengeStatus.SCHEDULED:
        return 'scheduled';
      default:
        return 'other';
    }
  }

  static ChallengeStatus fromName(String name) {
    switch (name) {
      case 'draft':
        return ChallengeStatus.DRAFT;
      case 'closed':
        return ChallengeStatus.CLOSED;
      case 'open':
        return ChallengeStatus.OPEN;
      case 'scheduled':
        return ChallengeStatus.SCHEDULED;
      default:
        return ChallengeStatus.OTHER;
    }
  }
}

enum ChallengeType {
  CHALLENGE,
  CAMPAIGN,
  OTHER;

  get name {
    switch (this) {
      case ChallengeType.CHALLENGE:
        return 'challenge';
      case ChallengeType.CAMPAIGN:
        return 'campaign';
      default:
        return 'other';
    }
  }

  static ChallengeType fromName(String name) {
    switch (name) {
      case 'challenge':
        return ChallengeType.CHALLENGE;
      case 'campaign':
        return ChallengeType.CAMPAIGN;
      default:
        return ChallengeType.OTHER;
    }
  }
}
